Unit ExecUtil;

{Contains utilities used by the executive (PasExec)}

Interface
Uses Crt, Printer, ExecVars, GenLib;

Var TraceOn : Boolean;                     
    TraceFile : Text;

Procedure Display(Blurb : String);
Procedure ShowEntDetails;
Procedure Schedule(Entity : Integer;
                   Activity : BAct;
                   RTime : Real);
Procedure Release(Entity : Integer);
Procedure InitEntFree(Nom : String);
Procedure InitEnt(Activity : BAct;
                  RTime : Real;
                  Nom : String);
Procedure AddC(CActivity : CAct);
Procedure ZeroLimits;
Function GetObsInt(X, Y : Integer) : Integer;
Procedure InterruptMessage;
Function GetDelayXY(X, Y : Integer) : Integer;
Procedure EventTraceQuery(X, Y : Integer);


Implementation

Procedure Display(Blurb : String);
{Writes to OpFile if Not Nul}
Begin
If TraceOn then
   Writeln(TraceFile, Blurb);
End;

Procedure ShowEntDetails;
{Writes current state of entity details to screen file or nul}
Var Entity : Integer;
Begin
If TraceOn then
   For Entity := 1 to NumEnts do
       With Details[Entity] do
            Begin
            Write(TraceFile, Entity, ':', Name, TimeCell:5);
            If Avail then Writeln(TraceFile, '  Avail')
            Else Writeln(TraceFile, '  Engaged');
            End;
End;

Procedure Schedule(Entity : Integer;
                   Activity : BAct;
                   RTime : Real);
{Commits specified entity to Activity after RTime}
Begin
With Details[Entity] do
     Begin
     Nextact := Activity;
     TimeCell := Clock + Round(RTime);
     Avail := False;
     Util := Util + Round(RTime);
     End;
End;

Procedure Release(Entity : Integer);
{Sets specified entity as Avail}
Begin
Details[Entity].Avail := True;
End;

Procedure InitEntFree(Nom : String);
{Creates new entity & sets this entity as initially free}
Begin
Inc(NumEnts);
With Details[NumEnts] do
     Begin
     Name := Nom;
     TimeCell := 0;
     Util := 0;
     End;
Release(NumEnts);
Display('Initialising avail entity ' + Int2Str(NumEnts,2));
End;

Procedure InitEnt(Activity : BAct;
                  RTime : Real;
                  Nom : String);
{Creates a new entity & sets up its intitial state}
Begin
InitEntFree(Nom);
Schedule(NumEnts, Activity, RTime);
Display('Initialising entity ' + Int2Str(NumEnts,2));
End;


Procedure AddC(CActivity : CAct);
Begin
Inc(NumCs);
CArray[NumCs] := CActivity;
End;

Procedure ZeroLimits;
{Sets Clock, NumEnts, NumCs to zero}
Begin
Clock := 0;                           {Set simulation clock to zero}
PrevClock := 0;                       {Set previous clock to zero}
NumEnts := 0;                         {No entities created yet}
NumCs := 0;                           {No Cs added yet}
End;


Function GetObsInt(X, Y : Integer) : Integer;
{Gets observation interval as 10, 20 or 30}
Var Value : Integer;
Begin
Value := 0;
Repeat
      GotoXY(X, Y);ClrEol;
      Write('Observation interval (10, 20 or 40):');
      Read(Value);
Until (Value in [10, 20, 40]);
GetObsInt := Value;
End;

Procedure InterruptMessage;
{Displays a message on-screen which invites a keypress to interrupt}
Begin
Gotoxy(22, 20);
Write(' ** PRESS ANY KEY TO CONTINUE ** ');
End;

Function GetDelayXY(X, Y : Integer) : Integer;
{Asks user to specify a delay to enable screen to be read on fast PCs}
Begin
GetDelayXY := GetIntXY(X, Y, 'Specify a display delay to run-time output', 0, 100);
End;

Procedure EventTraceQuery(X, Y : Integer);
{Asks user whether to produce an event trace file, at (X,Y)}
Var Ans : Char;
    FileName : String;
Begin
Ans := GetYNAnsXY(X, Y, 'Set up event trace file?');
If Ans in ['Y', 'y'] then
   Begin
   SetUpFileXY(X, Y, TraceFile, FileName);
   TraceOn := True;
   End
Else TraceOn := False;
End;


End.  {Of ExecUtil unit}