Unit GenLib;

Interface

Uses Crt;

Const NumStreams = 40;                     {Max No of RN streams}
Var RndSeed : Array[1..NumStreams,  0..1] of LongInt;  {RN Seeds}

Procedure Beep;
Procedure BeepClear(X, Y : Integer);
Procedure AnyKey;
Procedure AnyKeyXY(X, Y : Integer);
Procedure CheckStop(X, Y : Integer);
Function GetInt(Blurb : String;
                Min, Max : Integer) : Integer;
Function GetIntXY(X, Y : Integer;
                  Blurb : String;
                  Min, Max : Integer) : Integer;
Function GetYNAns(Blurb : String) : Char;
Function GetYNAnsXY(X, Y : Integer;
                    Blurb : String) : Char;
Procedure WriteTitle(Y : Integer;
                     Blurb : String);
Procedure Error(Message : String);
Procedure ClearFullScreen;

Procedure SetUpFileXY(X, Y : Integer;
                      Var LogicalFile : Text;
                      Var FileName : String);
Procedure OpenFileXY(X, Y : Integer;
                      Var LogicalFile : Text;
                      Var FileName : String);
Function Rnd(Stream : Integer) : Real;
Procedure InitStreams;
Function NegExp(Mean, Stream : Integer): Real;
Function Int2Str(Num, Len : Integer) : String;
Function Str2Int(Str : String) : Integer;

Implementation

Procedure Beep;
{Guess what this does}
Begin
Sound(300);
Delay(200);
NoSound;
End;

Procedure BeepClear(X, Y : Integer);
{Beeps and clears to the end of the line}
Begin
Gotoxy(X, Y);
Beep;
ClrEol;
End;

Procedure AnyKey;
{Writes message on next line, Beeps, waits for KeyPress}
Var Key : Char;
Begin
Beep;
Write('********  Press any key to continue  ********');
Key := ReadKey;
End;

Procedure AnyKeyXY(X, Y : Integer);
{Performs AnyKey at (X,Y)}
Begin
Gotoxy(X, Y);
AnyKey;
End;

Procedure CheckStop(X, Y : Integer);
{Checks to see if should make clean exit now}
Begin
BeepClear(X, Y);
Write('**  <ESC> to quit, other keys to continue  **');
If (Ord(ReadKey) = 27) then
   Halt(0);
End;


Procedure ReadIntAndCheck(Var Name : integer);
{Accepts keyboard input and checks for integer values. Beeps if non-
integer and waits until integer}
Var Blurb : String;
    x, y, Code : integer;
Begin
x := WhereX;                              {Current x co-ord of cursor}
y := WhereY;                              {Current y co-ord of cursor}
Repeat
      Gotoxy(x, y);
      Readln(Blurb);
      Val(Blurb, Name, Code);
      If Code <> 0 then
         BeepClear(x, y);
Until Code = 0;
End;


Function GetInt(Blurb : String;
                Min, Max : Integer) : Integer;
{Writes blurb and waits for response -> within range of Min..Max}
Var OK : Boolean;
    Num, x, y : Integer;
Begin
OK := False;
Repeat
      Write(Blurb+'(', Min, '..', Max, '): ');
      x := WhereX;
      y := WhereY;
      ReadIntAndCheck(Num);
      If (Num >= Min) and (Num <= Max) then OK := True
         Else BeepClear(x, y);
Until OK;
GetInt := Num;
End;

Function GetIntXY(X, Y : Integer;
                  Blurb : String;
                  Min, Max : Integer) : Integer;
{Writes blurb at X,Y and waits for response -> within range of Min..Max}
Var OK : Boolean;
    Num : Integer;
Begin
OK := False;
Repeat
      Gotoxy(X, Y);
      Write(Blurb+'(', Min, '..', Max, '): ');
      ReadIntAndCheck(Num);
      If (Num >= Min) and (Num <= Max) then OK := True
         Else BeepClear(X, Y);
Until OK;
GetIntXY := Num;
End;

Function GetYNAns(Blurb : String) : Char;
{Writes a question on-screen and waits for y, Y, n or N as response}
Var A : Char;
    x, y : Integer;
Begin
Write(Blurb + '(Y/N) ');
x := WhereX;
y := WhereY;
Repeat
      A := UpCase(ReadKey);
      Writeln(A);
      If Not (A In ['Y', 'N']) then BeepClear(x, y);
Until A in ['Y', 'N'];
GetYNAns := A;
End;

Function GetYNAnsXY(X, Y : Integer;
                    Blurb : String) : Char;
{Writes a question on-screen at X, Y and waits for y, Y, n or N as response}
Begin
Gotoxy(X,Y);
GetYNAnsXY := GetYNAns(Blurb);
End;

Procedure WriteTextXY(X, Y : Integer;
                      Blurb : String);
{Writes Blurb at (X, Y)}
Begin
Gotoxy(X, Y);
Write(Blurb);
End;

Procedure WriteTitle(Y : Integer;
                     Blurb : String);
{Centres Blurb on-screen at line Y}
Var X : Integer;
Begin
X := Round((80 - Length(Blurb))/2);
WriteTextXY(X, Y, Blurb);
End;

Procedure Error(Message : String);
{Writes an error message and halts the program. No recovery}
Begin
Gotoxy(1, 25);Writeln('FATAL ERROR: ', Message);
Halt(1);
End;

Procedure ClearFullScreen;
{Clears screen after key pressed}
Var Key : Char;
Begin
Beep;
WriteTextXY(30, 24, 'PRESS ANY KEY TO CONTINUE');
Key := ReadKey;
ClrScr;
End;
  

Procedure SetUpFileXY(X, Y : Integer;
                      Var LogicalFile : Text;
                      Var FileName : String);
{Creates a new text file for output}
Begin
Gotoxy(X, Y);ClrEol;
Write('File name? ');
Readln(FileName);
Assign(LogicalFile, FileName);
ReWrite(LogicalFile);
End;

Procedure OpenFileXY(X, Y : Integer;
                      Var LogicalFile : Text;
                      Var FileName : String);
{Assigns & opens a new text file for input}
Begin
Gotoxy(X, Y);ClrEol;
Write('File name? ');
Readln(FileName);
Assign(LogicalFile, FileName);
ReSet(LogicalFile);


End;
Function Rnd(Stream : Integer) : Real;
{Full 32 bit generator which utilises the overflow of Long Integers to
permit rapid division}
Const A = 16807;
      M = MaxLongInt;
Begin
RndSeed[Stream, 0] := (RndSeed[Stream, 1]*A) Mod M;
RndSeed[Stream, 1] := RndSeed[Stream, 0];
If RndSeed[Stream, 0] < 0 then Rnd := (M + RndSeed[Stream, 0])/M
   Else Rnd := RndSeed[Stream, 0]/M;
End;

Procedure InitStreams;
{Just one arbitrary way of initialising the random number streams. If run at
the start of a simulation, it will guarantee a particular set of streams.
Note that the seeds it produces are odd integers, because the generator is
multiplicative}
Var Loop : 1..NumStreams;
Begin
For Loop := 1 to Numstreams do
    Begin
    RndSeed[Loop, 1] := 10*Loop + 1;
    End;
End;

Function NegExp(Mean, Stream : Integer) : Real;
{Returns a real negative exponential random variate using Stream}
Var FMean : Real;
Begin
FMean := Mean;
NegExp := -1.0*FMean*ln(Rnd(Stream));
End;

Function Int2Str(Num, Len : Integer) : String;
{Converts Integer Num into a string of length Len}
Var Blurb : String;
Begin
Str(Num:Len, Blurb);
Int2Str := Blurb;
End;

Function Str2Int(Str : String) : Integer;
Var Code, X : Integer;
Begin
Val(Str, X, Code);
If Code <> 0 then Error(Str + ' is not an integer')
Else Str2Int := X;
End;



End.   {Of SimGen Unit}