{$F+}
Unit Model;
{Contains the constants, types and variables needed by the specific model,
together with the Bs, Cs, Initialisation & Finalisation plus associated
procedures & functions
Should be compatible with CSim and VBSIm libraries, started February 1997}

Interface
Uses ExecVars, ExecUtil, GenLib, Crt;

Procedure Initialisation;
Procedure Finalisation;
Procedure UpDateScreen;
Var DisplayDelay : Integer;

Implementation

Var {Variable definitions for activities ..etc..}
    PersEnq, PhoneEnq, Observer : Integer;
    Obs : Integer;
    PersArrTime, PhoneArrTime, PersServeTime, PhoneServeTime : Integer;
    PersIn, PhoneIn, PersQ, PhoneQ, PersOut, PhoneOut : Integer;
    PersRec, PhoneRec : Observations;
    ObsInterval : Integer;
    NumClerks : Integer;                   {Number of Clerks}
    FirstClerk, LastClerk : Integer;       {Check values}


{***************************************************************************
*****************************   Run-time stuff  ***************************
***************************************************************************}
Procedure ScreenBkGrnd(RunTime : Boolean);
{ Serves as a background for the run-time screen }
Begin
ClrScr;
WriteTitle(2, 'Harassed booking clerk simulation - Turbo Pascal version');
WriteTitle(3, '--------------------------------------------------------');

Gotoxy(20, 4);Write('Number of clerks: ', NumClerks);
Gotoxy(20, 6);Write('Simulation clock: ');

Gotoxy(20, 8);Write('PERSONAL ENQUIRERS');
Gotoxy(25, 9);Write('Arrivals:');
Gotoxy(25, 10);Write('Departures:');
Gotoxy(25, 11);Write('Queue lengths:');

Gotoxy(50, 8);Write('PHONE CALLS');
Gotoxy(55, 9);Write('Arrivals:');
Gotoxy(55, 10);Write('Departures:');
Gotoxy(55, 11);Write('Queue lengths:');

Gotoxy(40, 6);Write(Clock);

if (RunTime) then
   InterruptMessage;
End;


Procedure UpDateScreen;
{ Writes run-time data to screen on ScreenBkGrnd. Calls CheckInterrupt}
Begin
Gotoxy(40, 6);Write(Clock);

Gotoxy(40, 9);Write(PersIn);
Gotoxy(40, 10);Write(PersOut);
Gotoxy(40, 11);Write(PersQ);

Gotoxy(70, 9);Write(PhoneIn);
Gotoxy(70, 10);Write(PhoneOut);
Gotoxy(70, 11);Write(PhoneQ);
End;



{***************************************************************************
**********************   B Activities  *************************************
***************************************************************************}
Procedure Arrive;
{Arrival of personal enquirer}
Var Time : Real;
Begin
Inc(PersIn);    {Increment number of arrivals}
Time := NegExp(PersArrTime, 1);
Inc(PersQ);        {Increment queue length}
Display('Personal enquirer No. ' + Int2Str(PersIn,3) + ' arrives, queue = '+
                Int2Str(PersQ,3) + ' next due after ' + Int2Str(Round(Time),3));
Schedule(PersEnq, Arrive, Time); {Schedule next personal arrival}
End;

Procedure EndServe;
{End of personal service. CurrEnt is clerk who is released}
Begin
Inc(PersOut);    {Increment number of completed services}
Display('End of personal service:' + Int2Str(PersOut,3) +
             ' freed Clerk:' + Int2Str(CurrEnt,2));
End;

Procedure Call;
{Arrival of phone call}
Var Time : Real;
Begin
Inc(PhoneIn);  {Increment number of calls}
Time := NegExp(PhoneArrTime, 2);
Inc(PhoneQ);      {Increment queue length}
Display('Phone enquirer No. ' + Int2Str(PhoneIn,3) + ' arrives, queue = '+
                 Int2Str(PhoneQ,3) + ' next after ' + Int2Str(Round(Time),3));
Schedule(PhoneEnq, Call, Time);  {Schedule next phone call}
End;

Procedure EndTalk;
{End of phone conversation. CurrEnt is clerk who is released}
Begin
Inc(PhoneOut);    {Increment number of completed conversations}
Display('End of phone conversation:' + Int2Str(PhoneOut,3) +
                   ' freed Clerk: ' + Int2Str(CurrEnt,2));
End;

Procedure Observe;
{Collect time series of queue lengths}
Begin
PersRec[Obs] := PersQ;
PhoneRec[Obs] := PhoneQ;
Inc(Obs);
Schedule(Observer, Observe, ObsInterval);    {Schedule next observation}
Display('Recording, Personal queue = ' + Int2Str(PersQ,3) +
                   ', phone queue = ' + Int2Str(PhoneQ,3));
End;

{***************************************************************************
**********************   C Activities  *************************************
***************************************************************************}
Procedure BeginPersService;
{Serve first customer in the queue of personal enquirers}
Var Time : Real;
    Clerk : Integer;
Begin
Clerk := FirstClerk-1;
While (PersQ > 0) and (Clerk < LastClerk) do
      Begin
      Inc(Clerk);
      If Details[Clerk].Avail then
         Begin
         CStarted := True;
         Dec(PersQ);
         Time := NegExp(PersServeTime, 3);
         Schedule(Clerk, EndServe, Time);
         Display('Start of Personal service with Clerk:' + Int2Str(Clerk,2));
         End;
      End;
End;

Procedure BeginPhoneConv;
{Answer call of first caller in the call queue}
Var Time : Real;
    Clerk : Integer;
Begin
Clerk := FirstClerk - 1;
While (PhoneQ > 0) and (Clerk < LastClerk) do
      Begin
      Inc(Clerk);
      If Details[Clerk].Avail then
         Begin
         CStarted := True;
         Dec(PhoneQ);
         Time := NegExp(PhoneServeTime, 4);
         Schedule(Clerk, EndTalk, Time);
         Display('Start of Phone Service with Clerk: ' + Int2Str(Clerk,2));
         End;
      End;
End;


{***************************************************************************
**********************   Initialisation & Finalisation  ********************
***************************************************************************}
Procedure GetRunPars;
{Used to ask for the parameters of the simulation run.
Called by initialisation}
Var Ans : Char;
    FileName : String;
Begin
ClrScr;
WriteTitle(2, 'Harassed booking clerk simulation: Array based executive');
NumClerks := GetIntXY(20, 5, 'How many clerks?', 1, 5);

PersEnq := 1;                         {Personal enquirer arrival machine}
PhoneEnq := 2;                        {Phone enquirer arrival machine}
Observer := 3;                        {Observer entity}

FirstClerk := 4;             {Row of first clerk in Details array}
LastClerk := 3+NumClerks;    {Row of last clerk in Details array}

RunDuration := GetIntXY(20, 7, 'Simulation duration?', 20, 1440);

Gotoxy(20, 9);Write('Input mean inter-arrival times');
PersArrTime := GetIntXY(30, 10, 'Personal enquirers', 1, 50);
PhoneArrTime := GetIntXY(30, 11, 'Phone callers', 1, 50);

Gotoxy(20, 13);Write('Input mean service times');
PersServeTime := GetIntXY(30, 14, 'Personal enquirers', 1, 50);
PhoneServeTime := GetIntXY(30, 15, 'Phone callers', 1, 50);


ObsInterval := GetObsInt(20, 17);

DisplayDelay := GetDelayXY(20, 19);
EventTraceQuery(20, 21);
End;

Procedure InitCounters;
{Initialises the counters used by the model}
Begin
PersIn := 0;
PhoneIn := 0;
PersOut := 0;
PhoneOut := 0;
PersQ := 0;
PhoneQ := 0;
End;

Procedure InitObservations;
{Sets up time series for data collection}
Var Loop : Integer;
Begin
For Loop := 1 to 100 do
    PersRec[Loop] := 0;
PhoneRec := PersRec;
Obs := 1;
End;

Procedure Initialisation;
Var Clerk : Integer;
Begin
GetRunPars;                           {Get run parameters}
InitStreams;                          {Initialise random number streams}
ZeroLimits;                           {Set parameters to zero}
InitCounters;                         {Set counters to zero}
InitObservations;                     {Set up time series}

{Set up first arrivals & first observation of queue lengths}
InitEnt(Arrive, -12*Ln(Rnd(1)), 'Personal  ');
InitEnt(Call, -10*Ln(Rnd(2)), 'Phone     ');
InitEnt(Observe, 20, 'Observer  ');
For Clerk := FirstClerk to LastClerk do    {Set clerks initially idle}
    InitEntFree('Clerk:'+Int2Str(Clerk-FirstClerk+1, 1));
AddC(BeginPersService);
AddC(BeginPhoneConv);

ScreenBkGrnd(True);
End;

Procedure Finalisation;
Var Clerk, Counter, Time : Integer;
    Utilisation : Real;
Begin
If TraceOn then
   Close(TraceFile);

ScreenBkGrnd(False);            {Use run-time screen for final output}
UpDateScreen;

GotoXY(20, 12);Write('Input mean times were .. ');
GotoXY(25, 13);Write('Inter-arrivals:');
GotoXY(25, 14);Write('Services:');
GotoXY(40, 13);Write(PersArrTime);
GotoXY(70, 13);Write(PersServeTime);
GotoXY(40, 14);Write(PhoneArrTime);
GotoXY(70, 14);Write(PhoneServeTime);

GotoXY(20, 16);Write('Clerk percent utilisations were as follows .. ');
Clerk := FirstClerk - 1;
For Counter := 1 to NumClerks do
    Begin
    Inc(Clerk);
    Utilisation := 100.0*(1.0*Details[Clerk].Util)/(1.0*RunDuration);
    GotoXY(20, 16+ Counter);Writeln(Counter:5, Utilisation:8:0);
    End;

ClearFullSCreen;

Writeln;
Writeln('Time    Pers Queue     Phone Queue');
Writeln('----    ----------     -----------');
For Counter := 1 to Obs-1 do
    Begin
    Time := Counter*20;
    If WhereY = 24 then       {Avoid losing table before seeing it}
       ClearFullScreen;
    Writeln(Time:3, PersRec[Counter]:11, PhoneRec[Counter]:15);
    End;
Writeln;
Writeln('Simulation over after ', Clock);
End;

End.    {Of Model unit}